/**
  ******************************************************************************
  * @file    SSD1306.c
  * @author  Waveshare Team
  * @version V1.0
  * @date    19-July-2017
  * @brief   This file includes the OLED driver for SSD1306 display module

  ******************************************************************************
  * @attention
  *
  * THE PRESENT FIRMWARE WHICH IS FOR GUIDANCE ONLY AIMS AT PROVIDING CUSTOMERS
  * WITH CODING INFORMATION REGARDING THEIR PRODUCTS IN ORDER FOR THEM TO SAVE
  * TIME. AS A RESULT, WAVESHARE SHALL NOT BE HELD LIABLE FOR ANY
  * DIRECT, INDIRECT OR CONSEQUENTIAL DAMAGES WITH RESPECT TO ANY CLAIMS ARISING
  * FROM THE CONTENT OF SUCH FIRMWARE AND/OR THE USE MADE BY CUSTOMERS OF THE
  * CODING INFORMATION CONTAINED HEREIN IN CONNECTION WITH THEIR PRODUCTS.
  *
  * <h2><center>&copy; COPYRIGHT 2014 Waveshare</center></h2>
  ******************************************************************************
  */


#include "SSD1306.h"

uint8_t OLED_GRAM[128][8];

uint8_t SPI_WriteByte(uint8_t TxData, uint8_t Cmd)
{

    uint16_t Data = (Cmd)?0xFF:0x00;
        
      Data = (Data << 8) | TxData;
      
      
      
  
      return SPI_WriteFrames_Blocking(SPI1, (uint8_t*)&Data, sizeof(Data));
  
}

/**
  * @brief  Inserts a delay time.
  *         
  * @param  Time: specifies the delay time length
  *         
  * @retval  None
**/
void Delay_us(uint32_t Time)
{
	do
	{
		Time --;
	}while(Time != 0);
}

/**
  * @brief  writes an byte to OLED
  *         
  * @param  Data:  the Data will be write to OLED 
  * @param  Cmd:  the Cmd determines the Data is acted as a command or a data.
  *                  @arg 0: specifies the Data is a command which will be write to command reister.
  *                  @arg 0: specifies the Data is a data which will be stored at the GDDRAM.
  * @retval 
**/
void OLED_WriteByte(uint8_t Data, uint8_t Cmd)
{

  SPI_WriteByte(Data, Cmd);	
}

/**
  * @brief   sets the positions of the character that will  be displayed.
  *         
  * @param  X: specifies the X position.
  * @param Y: specifies the Y position.
  * @retval  None
**/

void OLED_SetPos(uint8_t X, uint8_t Y) 
{ 
	OLED_WriteByte(0xb0 + Y, OLED_CMD);
	OLED_WriteByte(((X & 0xf0) >> 4) | 0x10, OLED_CMD);
	OLED_WriteByte((X & 0x0f) | 0x01, OLED_CMD); 
}   	  


/**
  * @brief  Enables the Display 
  *         
  * @param  None
  *         
  * @retval  None
**/ 

void OLED_DisplayOn(void)
{

    OLED_WriteByte(0X8D, OLED_CMD);  
    OLED_WriteByte(0X14, OLED_CMD);  
    OLED_WriteByte(0XAF, OLED_CMD);  
}
   
/**
  * @brief  Disables the Display
  *          
  * @param  None
  *         
  * @retval  None
**/
void OLED_DisplayOff(void)
{
	OLED_WriteByte(0X8D, OLED_CMD);  
	OLED_WriteByte(0X10, OLED_CMD);  
	OLED_WriteByte(0XAE, OLED_CMD);  
}

/**
  * @brief   refresh the GDDRAM
  *         
  * @param  None
  *         
  * @retval  None
**/

void OLED_Refresh_Gram(void)
{
	uint8_t i, n;
	
	for(i = 0; i < 8; i ++)  
	{  
		OLED_WriteByte(0xb0 + i, OLED_CMD);  
		OLED_WriteByte(0x00, OLED_CMD);      
		OLED_WriteByte(0x10, OLED_CMD);         
		for(n = 0; n < 128; n ++)
		{
			OLED_WriteByte(OLED_GRAM[n][i], OLED_DAT); 
		}
	}   
}


/**
  * @brief   Clears the hole OLED.
  *         
  * @param  None
  *         
  * @retval None
**/

void OLED_ClearDisplay(void)  
{ 
	uint8_t i, n;
	
	for(i = 0; i < 8; i ++)
	{
		for(n = 0; n < 128; n ++)
		{
			OLED_GRAM[n][i] = 0x00;
		}
	}
	OLED_Refresh_Gram();
}
void OLED_SetDisplay(void)  
{ 
	uint8_t i, n;
	
	for(i = 0; i < 8; i ++)
	{
		for(n = 0; n < 128; n ++)
		{
			OLED_GRAM[n][i] = 0xff;
		}
	}
	OLED_Refresh_Gram();
}
	
/**
  * @brief  Displays a pixel. 
  *         
  * @param  X: pixel x.
  * @param  X: pixel y.
  * @param  Fill: wheher recerses the point.
  *  This paramter can ba one of following value;
  *  @arg Fill: where Fill can be 0 or 1
  *         
  * @retval None
**/

void OLED_DrawPoint(uint8_t X, uint8_t Y, uint8_t Fill)
{
	uint8_t Pos, Bx, Temp = 0;
	
	if(X > 127 || Y > 63)
		return;
	Pos = 7 - Y / 8;
	Bx = Y % 8;
	Temp = 1 << (7 - Bx);
	if(Fill)
	{
		OLED_GRAM[X][Pos] |= Temp;
	}
	else 
	{
		OLED_GRAM[X][Pos] &= ~Temp;	 
	}
}

	  
/**
  * @brief  Dispalys a full rectangle.
  *         
  * @param  X1: specifies the upper left corner X position of the rectangle
  * @param  Y1: specifies the upper left corner Y position of the rectangle
  * @param  X2: specifies the lower right corner X positon of the rectangle
  * @param  Y2: specifies the lower right corner Y positon of the rectangle
  *         
  * @retval  None
**/
void OLED_Fill(uint8_t X1, uint8_t Y1, uint8_t X2, uint8_t Y2, uint8_t Dot)  
{  
	uint8_t X, Y; 
	
	for(X = X1; X <= X2; X ++)
	{
		for(Y = Y1; Y <= Y2; Y ++)
			OLED_DrawPoint(X, Y, Dot);
	}													    
	//OLED_Refresh_Gram();
}

void OLED_View( uint8_t X1, uint8_t Y1, uint8_t X2, uint8_t Y2, const uint8_t Image[] ){


}


/**
  * @brief  Displays one character   
  *         
  * @param  X:
  *   This parameter can be one of the following values:
  *     @arg X: where x can be 0..127
  * @param  Y: 
  *   This parameter can be one of the following values:
  *     @arg Y: where y can be 0..63
  * @param  Chr: character ascii code,must be between 0x20 and 0x7e.
  * @param  Size: specifies the size of the font(12 or 16)
  * @param Mode: whether reverses the character
  *   This paramter can be one of following value;
  *   @arg Mode: where Mode can be 0 or 1
  * @retval None
**/
#if 1
void OLED_ShowChar(uint8_t X, uint8_t Y, uint8_t Chr, uint8_t Size, uint8_t Mode)
{      	
	uint8_t Temp, t, t1;
	uint8_t y0 = Y;
	
	Chr = Chr - ' ';				   
    for(t = 0; t < Size; t ++)
    {   
		if(Size == 12)
		{
			Temp = oled_asc2_1206[Chr][t];  
		}
		else 
		{
			Temp = oled_asc2_1608[Chr][t];
		}
        for(t1 = 0; t1 < 8; t1 ++)
		{
			if(Temp & 0x80)
				OLED_DrawPoint(X, Y, Mode);
			else 
				OLED_DrawPoint(X, Y, !Mode);
			Temp <<= 1;
			Y ++;
			if((Y - y0) == Size)
			{
				Y = y0;
				X ++;
				break;
			}
		}  	 
    } 
}
#endif


/**
  * @brief  Displays a string on the OLED
  *         
  * @param  X:  specifies X position at which a string will be to display 
  * @param  Y:  specifies Y position at which a string will be to display 
  * @param  Str: pointer to a string to dispaly on OLED. 
  *         
  * @retval  None
**/
#if 1
void OLED_ShowString(uint8_t X, uint8_t Y, const uint8_t *Str)
{
#define MAX_CHAR_POSX 122
#define MAX_CHAR_POSY 58
	
    while(*Str != '\0')
    {       
        if(X > MAX_CHAR_POSX)
		{
			X = 0;
			Y += 16;
		}
        if(Y > MAX_CHAR_POSY)
		{
			Y = X = 0;
			OLED_ClearDisplay();
		}
        OLED_ShowChar(X, Y, *Str, 16, 1);	 
        X += 8;
        Str ++;
    } 
}

#endif

void OLED_SetPageAddress(uint32_t PageOffSet){
  
  OLED_WriteByte(0x22, OLED_CMD);//-set display offset	Shift Mapping RAM Counter (0x00~0x3F)
  OLED_WriteByte(PageOffSet, OLED_CMD);//-not offset
  
}

void OLED_StartLine(uint32_t StartLine){
  
  OLED_WriteByte(0xD3, OLED_CMD);//-set display offset	Shift Mapping RAM Counter (0x00~0x3F)
  OLED_WriteByte(StartLine, OLED_CMD);//-not offset
  
}

void OLED_OffSet(uint32_t OffSet){
  
  OLED_WriteByte(0xD3, OLED_CMD);//-set display offset	Shift Mapping RAM Counter (0x00~0x3F)
  OLED_WriteByte(OffSet, OLED_CMD);//-not offset
  
}

void OLED_SetDisplayEnable(bool TurnOn){
  OLED_WriteByte((TurnOn)?0xAF:0xAE, OLED_CMD);
}

void OLED_SetHorizontalScroll(SCROL_MODE Mode){
  
  //OLED_WriteByte(0xAE, OLED_CMD);
  
  switch(Mode){
    
    case LeftMode:
      
    case RightMode:
      
      OLED_WriteByte(0x2E, OLED_CMD);                        //  
  
      OLED_WriteByte((Mode==LeftMode)?0x27:0x26, OLED_CMD);  //    27 - , 26 - 
      
      OLED_WriteByte(0x00, OLED_CMD);                        //  
      
      OLED_WriteByte(0x02, OLED_CMD);//OLED_WriteByte((Mode==LeftMode)?0x00:0x07, OLED_CMD);  //   000b  PAGE0, 001b  PAGE1, 010b  PAGE2, 011b  PAGE3, 100b  PAGE4, 101b  PAGE5, 110b  PAGE6, 111b  PAGE7
      
      OLED_WriteByte(0x07, OLED_CMD);                        //      ,    000b  5 frames, 001b  64 frames, 010b  128 frames, 011b  256 frames, 100b  3 frames, 101b  4 frames, 110b  25 frame, 111b  2 frame
      
      OLED_WriteByte(0x07, OLED_CMD);//OLED_WriteByte((Mode==LeftMode)?0x07:0x00, OLED_CMD);  //      
      
      OLED_WriteByte(0x00, OLED_CMD);                        //  
      
      OLED_WriteByte(0xFF, OLED_CMD);                        // FF 
      
      OLED_WriteByte(0x2F, OLED_CMD);                        //  
    
    break;
  
    case OffMode:
      OLED_WriteByte(0x2E, OLED_CMD);
    break;  
    
    case OnMode:
      OLED_WriteByte(0x2F, OLED_CMD);
    break;    
  }
  
  //OLED_WriteByte(0xAF, OLED_CMD);
  
}

/**
  * @brief  initializes the OLED.
  *         
  * @param  None
  *         
  * @retval  None
**/
void OLED_Init(void)
{

  OLED_DisplayOff();
  
  Delay_us(5000);
  
	OLED_WriteByte(0xAE, OLED_CMD);//--turn off oled panel
	OLED_WriteByte(0x00, OLED_CMD);//---set low column address
	OLED_WriteByte(0x10, OLED_CMD);//---set high column address
	OLED_WriteByte(0x40, OLED_CMD);//--set start line address  Set Mapping RAM Display Start Line (0x00~0x3F)
	OLED_WriteByte(0x81, OLED_CMD);//--set contrast control register
	OLED_WriteByte(0xCF, OLED_CMD); // Set SEG Output Current Brightness
	OLED_WriteByte(0xA0, OLED_CMD);//--Set SEG/Column Mapping     
	OLED_WriteByte(0xC8, OLED_CMD);//Set COM/Row Scan Direction   
	OLED_WriteByte(0xA6, OLED_CMD);//--set normal display
	OLED_WriteByte(0xA8, OLED_CMD);//--set multiplex ratio(1 to 64)
	OLED_WriteByte(0x3f, OLED_CMD);//--1/64 duty
	OLED_WriteByte(0xD3, OLED_CMD);//-set display offset	Shift Mapping RAM Counter (0x00~0x3F)
	OLED_WriteByte(0x00, OLED_CMD);//-not offset
	OLED_WriteByte(0xd5, OLED_CMD);//--set display clock divide ratio/oscillator frequency
	OLED_WriteByte(0x80, OLED_CMD);//--set divide ratio, Set Clock as 100 Frames/Sec
	OLED_WriteByte(0xD9, OLED_CMD);//--set pre-charge period
	OLED_WriteByte(0xF1, OLED_CMD);//Set Pre-Charge as 15 Clocks & Discharge as 1 Clock
	OLED_WriteByte(0xDA, OLED_CMD);//--set com pins hardware configuration
	OLED_WriteByte(0x12, OLED_CMD);
	OLED_WriteByte(0xDB, OLED_CMD);//--set vcomh
	OLED_WriteByte(0x40, OLED_CMD);//Set VCOM Deselect Level
	OLED_WriteByte(0x20, OLED_CMD);//-Set Page Addressing Mode (0x00/0x01/0x02)
	OLED_WriteByte(0x02, OLED_CMD);//
	OLED_WriteByte(0x8D, OLED_CMD);//--set Charge Pump enable/disable
	OLED_WriteByte(0x14, OLED_CMD);//--set(0x10) disable
	OLED_WriteByte(0xA4, OLED_CMD);// Disable Entire Display On (0xa4/0xa5)
	OLED_WriteByte(0xA6, OLED_CMD);// Disable Inverse Display On (0xa6/a7) 
	OLED_WriteByte(0xAF, OLED_CMD);//--turn on oled panel
  
  Delay_us(5000);
  
  OLED_DisplayOn();
  
  
  OLED_ClearDisplay();
  

}

/******************* (C) COPYRIGHT 2014 Waveshare *****END OF FILE*******************/

